/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core;

import java.io.Serializable;

/**
 * This class represents the properties of boolean parameter groups. A boolean
 * parameter group has two states, it can be either set or unset. The enabled
 * and defaultValueEnabled attribute of the super class are used to manage its
 * state and default value.
 * 
 */

public class BooleanParameterGroup extends ParameterGroup implements
		Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates the parameter group according to the passed values.
	 */
	public BooleanParameterGroup(String name, String key, boolean defaultValue,
			String description, String successor, boolean visible) {
		super(name, key, description, successor, visible, defaultValue);
	}

	@Override
	public String getDisplayFriendlyValueString() {
		if (this.enabled)
			return "Set";
		else
			return "Unset";
	}

	@Override
	public String getDescription() {
		return this.description;
	}

	@Override
	public String getCommandlineCallString() {
		if (this.enabled)
			// For boolean parameter groups the command line call consists just
			// of the key
			return this.key;
		else
			return "";
	}

}
