/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.BooleanParameterGroup;

/**
 * This class defines the history object used to change the value of a
 * BooleanParameterGroup.
 */

public class ChangeBooleanParameterGroupValuesHistoryObject extends
		HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the BooleanParameterGroup that will be changed.
	 */
	private BooleanParameterGroup group;
	
	/**
	 * Represents the newValue whether the ParameterGroup is enabled.
	 */
	private boolean newValue;
	
	/**
	 * Represents the oldValue whether the ParameterGroup is enabled.
	 */
	private boolean oldValue;
	
	/**
	 * Represents the name of the node.
	 */
	private String nodeName;

	/**
	 * Constructor for a ChangeBooleanParameterGroupValuesHistoryObject.
	 */
	public ChangeBooleanParameterGroupValuesHistoryObject(String nodeName,
			BooleanParameterGroup booleanParameterGroup, boolean newValue) {
		this.group = booleanParameterGroup;
		this.newValue = newValue;
		this.oldValue = this.group.isEnabled();
		this.nodeName = nodeName;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeBooleanParameterGroupValuesHistoryObject(
				this.nodeName, this.group, this.oldValue);
	}

	@Override
	public String getInformationString() {
		if (this.newValue)
			return String.format(HistoryStrings.ChangeBooleanParameterGroupValuesString, "Set", this.group.getName(), this.nodeName);
		else
			return String.format(HistoryStrings.ChangeBooleanParameterGroupValuesString, "Unset", this.group.getName(), this.nodeName);

	}

	@Override
	public void performAction() {
		this.group.setEnable(this.newValue);
	}

}
