package core.history;

import helper.SmallHelpers;
import core.graph.GraphNode;

/**
 * This class defines the history object used to change a node comment.
 */
public class ChangeNodeCommentHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the GraphNode that will be changed.
	 */
	private GraphNode abstractNode;
	
	/**
	 * Represents the oldComment.
	 */
	private String oldComment;
	
	/**
	 * Represents the newComment.
	 */
	private String newComment;

	/**
	 * Constructor for a ChangeNodeCommentHistoryObject.
	 */
	public ChangeNodeCommentHistoryObject(GraphNode abstractNode,
			String newComment) {
		this.abstractNode = abstractNode;
		this.oldComment = abstractNode.getComment();
		this.newComment = SmallHelpers.removeAllInvalidChars(newComment, "",
				"Comment");
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeNodeCommentHistoryObject(this.abstractNode,
				this.oldComment);
	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.ChangeNodeCommentString, this.abstractNode.getName());
	}

	@Override
	public void performAction() {
		this.abstractNode.setComment(this.newComment);
	}

}
