package core.history;

import core.graph.GraphNode;

/**
 * This class defines the history object used to change the visibility of the comment for a node.
 */
public class ChangeNodeCommentVisibiltyHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the GraphNode that will get changed.
	 */
	private GraphNode abstractNode;
	
	/**
	 * oldValue of the visibility.
	 */
	private boolean oldValue;
	
	/**
	 * newValue of the visibility.
	 */
	private boolean newValue;

	/**
	 * Constructor for a ChangeNodeCommentVisibilityHistoryObject.
	 */
	public ChangeNodeCommentVisibiltyHistoryObject(GraphNode abstractNode,
			boolean newValue) {
		this.abstractNode = abstractNode;
		this.oldValue = abstractNode.isCommentVisible();
		this.newValue = newValue;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeNodeCommentVisibiltyHistoryObject(this.abstractNode,
				this.oldValue);
	}

	@Override
	public String getInformationString() {	
		if (this.newValue)
			return String.format(HistoryStrings.ChangeNodeCommentVisibiltyString, "Enable", this.abstractNode.getName());
		else
			return String.format(HistoryStrings.ChangeNodeCommentVisibiltyString, "Disable", this.abstractNode.getName());

	}

	@Override
	public void performAction() {
		this.abstractNode.setCommentVisibilty(this.newValue);
	}

}
