/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import helper.SmallHelpers;
import core.graph.GraphNode;

/**
 * This class defines the history object used to change the name of a node.
 */

public class ChangeNodeNameHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the GraphNode that will be renamed.
	 */
	private GraphNode abstractNode;
	
	/**
	 * Stores the oldName of the GraphNode.
	 */
	private String oldName;
	
	/**
	 * Represents the newName for the GraphNode.
	 */
	private String newName;

	/**
	 * This constructor creates a ChangeNodeNameHistoryObject.
	 */
	public ChangeNodeNameHistoryObject(GraphNode abstractNode, String newName) {
		this.abstractNode = abstractNode;
		this.oldName = abstractNode.getName();
		this.newName = SmallHelpers.removeAllInvalidChars(newName, "",
				"Node Name");
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeNodeNameHistoryObject(this.abstractNode, this.oldName);
	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.ChangeNodeNameString, this.oldName, this.newName);
	}

	@Override
	public void performAction() {
		this.abstractNode.setName(this.newName);
	}
}
