/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.ParameterGroup;

/**
 * This class defines the history object used to set or unset the enabled value
 * of a ParameterGroup.
 */

public class ChangeParameterGroupEnable extends HistoryObject {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the ParameterGroup that will be changed.
	 */
	private ParameterGroup group;
	
	/**
	 * Represents the newValue whether the ParameterGroup is enabled or
	 * disabled.
	 */
	private boolean newValue;
	
	/**
	 * Stores the oldValue.
	 */
	private boolean oldValue;
	
	/**
	 * Representation of the node name.
	 */
	private String nodeName;

	public ChangeParameterGroupEnable(String nodeName,
			ParameterGroup parameterGroup, boolean newValue) {
		this.group = parameterGroup;
		this.newValue = newValue;
		this.oldValue = this.group.isEnabled();
		this.nodeName = nodeName;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeParameterGroupEnable(this.nodeName, this.group,
				this.oldValue);
	}

	@Override
	public String getInformationString() {
		if (this.newValue)
			return String.format(HistoryStrings.ChangeParameterGroupEnableString, "Enable", this.group.getName(), this.nodeName);
		else
			return String.format(HistoryStrings.ChangeParameterGroupEnableString, "Disable", this.group.getName(), this.nodeName);
	}

	@Override
	public void performAction() {
		this.group.setEnable(this.newValue);
	}
}
