/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.ParameterGroup;

/**
 * This class defines the history object used to change the visibility of a
 * ParameterGroup.
 */

public class ChangeParameterGroupVisibility extends HistoryObject {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the ParameterGroup.
	 */
	private ParameterGroup group;
	
	/**
	 * newValue for the visibility of the ParameterGroup.
	 */
	private boolean newValue;
	
	/**
	 * stores the oldValue of the visibility.
	 */
	private boolean oldValue;
	
	/**
	 * represents the name of the node.
	 */
	private String nodeName;

	/**
	 * This constructor creates a ChangeParameterGroupVisibilityHistoryObject.
	 */
	public ChangeParameterGroupVisibility(String nodeName,
			ParameterGroup textParameterGroup, boolean newValue) {
		this.group = textParameterGroup;
		this.newValue = newValue;
		this.oldValue = this.group.isVisible();
		this.nodeName = nodeName;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeParameterGroupVisibility(this.nodeName, this.group,
				this.oldValue);
	}

	@Override
	public String getInformationString() {		
		if (this.newValue)
			return String.format(HistoryStrings.ChangeParameterGroupVisibilityString, "Enable", this.group.getName(), this.nodeName);
		else
			return String.format(HistoryStrings.ChangeParameterGroupVisibilityString, "Disable", this.group.getName(), this.nodeName);
	}

	@Override
	public void performAction() {
		this.group.setVisible(this.newValue);
	}
}
