/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.graph.PointNode;

/**
 * This class defines the history object used to change whether a PointNode is user positioned. 
 */
public class ChangePointNodeUserPositioned extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the pointNode that will be changed.
	 */
	private PointNode pointNode;
	
	/**
	 * represents the new state of the pointNode.
	 */
	private boolean newState;
	
	/**
	 * represents the old state of the pointNode.
	 */
	private boolean oldState;

	/**
	 * This constructor creates a ChangePointNodeUserPositioned for given pointNode
	 * and the newState for that pointNode.
	 */
	public ChangePointNodeUserPositioned(PointNode pointNode, boolean newState) {
		super();
		this.pointNode = pointNode;
		this.newState = newState;
		this.oldState = pointNode.isUserPositioned();

	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangePointNodeUserPositioned(this.pointNode, this.oldState);
	}

	@Override
	public void performAction() {
		this.pointNode.setUserPositioned(this.newState);

	}

	@Override
	public String getInformationString() {
		if (this.newState == true) {
			return String.format(HistoryStrings.ChangePointNodeUserPositionedString, "Disable", this.pointNode.getEdge().getDisplayFriendlyName());
		} else {
			return String.format(HistoryStrings.ChangePointNodeUserPositionedString, "Enable", this.pointNode.getEdge().getDisplayFriendlyName());
		}
	}

}
