/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.SelectParameterGroup;
import core.graph.JobNode;

/**
 * This class defines the history object used to change the selected value of a
 * SelectParameterGroup.
 */

public class ChangeSelectParameterGroupValuesHistoryObject extends
		HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the SelectParameterGroup that will be changed.
	 */
	private SelectParameterGroup group;
	
	/**
	 * Represents the new selected ParameterGroup.
	 */
	private String newChoice;
	
	/**
	 * Stores the old selected ParameterGroup.
	 */
	private String oldChoice;
	
	/**
	 * Reference to the JobNode.
	 */
	private JobNode node;

	/**
	 * Creates a ChangeSelectParameterGroupValuesHistoryObject
	 */
	public ChangeSelectParameterGroupValuesHistoryObject(JobNode node,
			SelectParameterGroup selectParameterGroup, String newChoice,
			String oldChoice) {
		this.group = selectParameterGroup;
		this.newChoice = newChoice;
		this.oldChoice = oldChoice;
		this.node = node;
	}

	@Override
	public HistoryObject createUndoObject() {

		return new ChangeSelectParameterGroupValuesHistoryObject(this.node,
				this.group, this.oldChoice, this.newChoice);
	}

	@Override
	public void performAction() {
		this.group.setChoiceValue(this.newChoice);

	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.ChangeSelectParameterGroupValuesString, this.group.getName(), this.node.getName(), this.oldChoice, this.newChoice);
	}
}
