/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import helper.SmallHelpers;
import core.StaticTextParameter;

/**
 * This class defines the history object used to change the value of one static
 * text parameter.
 */

public class ChangeStaticTextParameterValueHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the StaticTextParameter that will be changed.
	 */
	private StaticTextParameter parameter;
	
	/**
	 * Represents the new value.
	 */
	private String newValue;
	
	/**
	 * Stores the old value.
	 */
	private String oldValue;

	/**
	 * This constructor creates a ChangeStaticTextParameterValueHistoryObject for a given StaticParameter and the newValue.
	 */
	public ChangeStaticTextParameterValueHistoryObject(
			StaticTextParameter staticParameter, String newValue) {
		this.parameter = staticParameter;
		this.newValue = SmallHelpers.removeAllInvalidChars(newValue, "",
				"Text Parameter");
		this.oldValue = this.parameter.getValue();
	}

	@Override
	public HistoryObject createUndoObject() {
		return new ChangeStaticTextParameterValueHistoryObject(this.parameter,
				this.oldValue);
	}

	@Override
	public String getInformationString() {
		// return "Change Text Parameter (" + parameter.getName() +
		// ") of Node (" + node.getName() + ") from " + oldValue + " to " +
		// newValue;
		return this.newValue + " -> " + this.oldValue;
	}

	@Override
	public void performAction() {
		this.parameter.setValue(this.newValue);
	}
}
