/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import java.io.Serializable;

/**
 * This class defines the properties of a history object.
 * 
 * A HistoryObject represents one change done to the workflow or an element in
 * it. The specific subclasses represent all changes that can be made. Any
 * modification should be done by instances of this class.
 */

public abstract class HistoryObject implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates and returns a HistoryObject undoing any change that is done by
	 * this HistoryObject.
	 */
	public abstract HistoryObject createUndoObject();

	/**
	 * Performs the action specified by this history object.
	 */
	public abstract void performAction();

	/**
	 * Returns a string describing the change that is done by this
	 * HistoryObject.
	 */
	public abstract String getInformationString();

}
