/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import java.util.Vector;

import core.graph.Edge;
import core.graph.Graph;
import core.graph.GraphNode;
import core.graph.InputFileNode;
import core.graph.OutputFileNode;

/**
 * This class defines the history object used to add a new node to the graph.
 */

public class NewNodeHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the new GraphNode.
	 */
	private GraphNode node;
	
	/**
	 * Stores all edges connected to the Node.
	 */
	private Vector<Edge> edges;
	
	/**
	 * Reference to the graph.
	 */
	private Graph graph;

	/**
	 * This constructor creates a NewNodeHistoryObject for a given GraphNode,
	 * graph and a vector of edges.
	 */
	public NewNodeHistoryObject(GraphNode abstractNode, Vector<Edge> edges,
			Graph graph) {
		this.graph = graph;
		this.node = abstractNode;
		this.edges = edges;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new RemoveNodeHistoryObject(this.node, this.graph);
	}

	@Override
	public String getInformationString() {		
		if (this.node instanceof InputFileNode)
			return String.format(HistoryStrings.NewNodeString, "Input Node", ((InputFileNode) this.node).getExternalFileName().getName());
		else if (this.node instanceof OutputFileNode)
			return String.format(HistoryStrings.NewNodeString, "Output Node", ((OutputFileNode) this.node).getExternalFileName().getName());
		else
			return String.format(HistoryStrings.NewNodeString, "Node", this.node.getName());
	}

	@Override
	public void performAction() {
		this.graph.addNode(this.node);
		if (this.edges != null) {
			for (Edge curEdge : this.edges) {
				this.graph.addEdge(curEdge);
			}
		}
	}

}
