/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import java.awt.Point;

import core.graph.BasicNode;
import core.graph.Edge;
import core.graph.Graph;
import core.graph.PointNode;

/**
 * This class defines the history object used to create a new PointNode.
 */
public class NewPointNodeHistoryObject extends HistoryObject {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the edge the new PointNode should be placed.
	 */
	private Edge pointNodesEdge;
	
	/**
	 * Reference to the PointNode that lays before the new created PointNode.
	 */
	private PointNode nodeBeforePointNode;
	
	/**
	 * Reference to the newPointNode.
	 */
	private PointNode newPointNode;
	
	/**
	 * Represents the coordinates for the new PointNode.
	 */
	private Point pointNodeCoordinates;
	
	/**
	 * Reference to the graph.
	 */
	private Graph graph;

	/**
	 * This constructor is used to create a new PointNode after a specified one
	 * from scratch.
	 */
	public NewPointNodeHistoryObject(Edge pointNodesEdge,
			BasicNode nodeBeforePointNode, Graph graph) {
		this.graph = graph;
		this.pointNodesEdge = pointNodesEdge;
		if (nodeBeforePointNode instanceof PointNode) {
			this.nodeBeforePointNode = (PointNode) nodeBeforePointNode;
		} else {
			this.nodeBeforePointNode = null;
		}
		this.pointNodeCoordinates = null;
	}

	/**
	 * This constructor is used to create a new PointNode after a specified one
	 * with specified grid coordinates.
	 */
	public NewPointNodeHistoryObject(Edge pointNodesEdge,
			PointNode nodeBeforePointNode, Point gridCoordinates, Graph graph) {
		this(pointNodesEdge, nodeBeforePointNode, graph);
		this.pointNodeCoordinates = gridCoordinates;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new RemoveNodeHistoryObject(this.newPointNode, this.graph);
	}

	@Override
	public void performAction() {
		this.newPointNode = this.pointNodesEdge
				.createNewPointNodeAfterSelected(this.nodeBeforePointNode,
						this.pointNodeCoordinates);
	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.NewPointNodeString, this.newPointNode.getEdge().getDisplayFriendlyName());
	}

}
