/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import java.util.Vector;

import core.graph.BasicNode;
import core.graph.Edge;
import core.graph.Graph;
import core.graph.GraphNode;
import core.graph.InputFileNode;
import core.graph.OutputFileNode;
import core.graph.PointNode;

/**
 * This class defines the history object used to remove a node from the graph.
 */
public class RemoveNodeHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the BasicNode that should be removed.
	 */
	private BasicNode node;
	
	/**
	 * Stores all edges that are connected to the BasicNode.
	 */
	private Vector<Edge> connectedEdges;
	
	/**
	 * Reference to the Graph.
	 */
	private Graph graph;
	
	/**
	 * This constructor creates a RemoveNodeHistoryObject for a given node and graph.
	 */
	public RemoveNodeHistoryObject(BasicNode node, Graph graph) {
		this.graph = graph;
		this.node = node;
		this.connectedEdges = graph.getAllEdgesConnectedToNode(node);
	}

	@Override
	public HistoryObject createUndoObject() {
		if (this.node instanceof PointNode)
			return new NewPointNodeHistoryObject(
					((PointNode) this.node).getEdge(),
					((PointNode) this.node).getLeftNeighbour(), this.graph);
		else
			return new NewNodeHistoryObject((GraphNode) this.node,
					this.connectedEdges, this.graph);
	}

	@Override
	public String getInformationString() {			
		if (this.node instanceof PointNode)
			return String.format(HistoryStrings.RemoveNodeString, "EdgePoint", ((PointNode) this.node).getEdge().getDisplayFriendlyName());
		else if (this.node instanceof InputFileNode)
			return String.format(HistoryStrings.RemoveNodeString, "Input Node", ((InputFileNode) this.node).getExternalFileName().getName());
		else if (this.node instanceof OutputFileNode)
			return String.format(HistoryStrings.RemoveNodeString, "Output Node", ((OutputFileNode) this.node).getExternalFileName().getName());
		else
			return String.format(HistoryStrings.RemoveNodeString, "Node", this.node.getName());
	}

	@Override
	public void performAction() {
		this.graph.removeNode(this.node);

		for (Edge curEdge : this.connectedEdges) {
			this.graph.removeEdge(curEdge);
		}
	}

}
