/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import core.graph.FileNode;
import core.graph.Graph;
import core.graph.InputFileNode;

/**
 * This class defines the history object used to set a converter Id.
 */
public class SetConverterIDHistoryObject extends HistoryObject {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Representation of the old ConverterID.
	 */
	private int oldConverterID;
	
	/**
	 * Repesentation of the new ConverterID.
	 */
	private int newConverterID;
	
	/**
	 * Reference to the FileNode.
	 */
	private FileNode node;
	
	/**
	 * Reference to the Graph.
	 */
	private Graph graph;

	/**
	 * This constructor creates a SetConverterIDHistoryObject for a given FilenNode, Graph and
	 * the new ConverterID.
	 */
	public SetConverterIDHistoryObject(FileNode markedAbstractNode,
			int newConverterID, Graph graph) {
		this.graph = graph;
		this.oldConverterID = markedAbstractNode.getConverterID();
		this.newConverterID = newConverterID;
		this.node = markedAbstractNode;

	}

	@Override
	public HistoryObject createUndoObject() {
		return new SetConverterIDHistoryObject(this.node, this.oldConverterID,
				this.graph);
	}

	@Override
	public void performAction() {
		this.node.setConverterID(this.newConverterID);
	}

	@Override
	public String getInformationString() {		
		if (this.node instanceof InputFileNode)
			return String.format(HistoryStrings.SetConverterIDString, "Input", this.node.getInformativeName(), this.graph.getReader().getInputConverterInputTypeByID(this.oldConverterID), this.graph.getReader().getInputConverterInputTypeByID(this.newConverterID));
		else
			return String.format(HistoryStrings.SetConverterIDString, "Output", this.node.getInformativeName(), this.graph.getReader().getOutputConverterOutputTypeByID(this.oldConverterID), this.graph.getReader().getOutputConverterOutputTypeByID(this.newConverterID));
	}

}
