/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.history;

import core.graph.Edge;

/**
 * This class defines the history object used to set the edge order key.
 */
public class SetEdgeOrderKeyHistoryObject extends HistoryObject {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the edge.
	 */
	private Edge edge;
	
	/**
	 * The old order key.
	 */
	private String oldKey;
	
	/**
	 * The new order key for the edge.
	 */
	private String newKey;

	/**
	 * This constructor creates a SetEdgeOrderKeyHistoryObject for a given edge and a newKey.
	 */
	public SetEdgeOrderKeyHistoryObject(Edge edge, String newKey) {
		this.edge = edge;
		this.oldKey = edge.getOrderKey();
		this.newKey = newKey;
	}

	@Override
	public HistoryObject createUndoObject() {
		return new SetEdgeOrderKeyHistoryObject(this.edge, this.oldKey);
	}

	@Override
	public void performAction() {
		this.edge.setOrderKey(this.newKey);
	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.SetEdgeOrderKeyString, this.edge.getDisplayFriendlyName(), this.oldKey, this.newKey);
	}

}
