/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import helper.SmallHelpers;
import core.graph.FileNode;
import core.graph.InputFileNode;

/**
 * This class defines the history object used to set a nodes the external
 * FileName and the ConverterID.
 */

public class SetExternalFileNameHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;
	
	/**
	 * newFileName for the external File.
	 */
	private String newFileName;
	
	/**
	 * oldFileName of the external File.
	 */
	private String oldFileName;
	
	/**
	 * Reference to the FileNode.
	 */
	private FileNode node;

	/**
	 * This constructor creates a SetExternalFileNameHistoryObject for a given node and newFileName.
	 */
	public SetExternalFileNameHistoryObject(FileNode node, String newFileName) {
		this.node = node;
		this.oldFileName = node.getExternalFileName().getPath();
		this.newFileName = SmallHelpers.removeAllInvalidChars(newFileName,
				"[$]", "Filename");
	}

	@Override
	public HistoryObject createUndoObject() {
		return new SetExternalFileNameHistoryObject(this.node, this.oldFileName);
	}

	@Override
	public String getInformationString() {
		if (this.node instanceof InputFileNode)
			return String.format(HistoryStrings.SetExternalFileNameString, "Input", this.oldFileName, this.newFileName);
		else
			return String.format(HistoryStrings.SetExternalFileNameString, "Output", this.oldFileName, this.newFileName);
	
	}

	@Override
	public void performAction() {
		this.node.setExternalFileName(this.newFileName);
	}
}
