/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.history;

import helper.SmallHelpers;
import core.graph.Edge;

/**
 * This class defines the history object used to set a file name.
 */
public class SetFileNameHistoryObject extends HistoryObject {

	private static final long serialVersionUID = 1L;

	/**
	 * newName of the FileName.
	 */
	private String newName;
	
	/**
	 * oldName of the FileName.
	 */
	private String oldName;
	
	/**
	 * Reference to the edge that represents the File.
	 */
	private Edge edge;

	/**
	 * This constructor creates a SetFileNameHistoryObject for a given edge and newName. 
	 */
	public SetFileNameHistoryObject(Edge edge, String newName) {
		this.edge = edge;
		this.oldName = edge.getFileName().getPath();
		this.newName = SmallHelpers.removeAllInvalidChars(newName, "[\\$\\:]",
				"Filename");
	}

	@Override
	public HistoryObject createUndoObject() {
		return new SetFileNameHistoryObject(this.edge, this.oldName);
	}

	@Override
	public void performAction() {
		this.edge.getFrom().setXMLFileName(this.newName);

	}

	@Override
	public String getInformationString() {
		return String.format(HistoryStrings.SetFileNameString, this.edge.getDisplayFriendlyName(), this.oldName, this.newName);
	}

}
