/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/**
 * This class defines the properties of the "About ProFiDo" frame.
 * 
 * It creates the frame which is shown after a click on the "About ProFiDo"
 * item.
 */
public class AboutFrame extends JFrame {
	// TODO Improve design, use ShowTextPanel (or create subclass)

	private static final long serialVersionUID = 1L;
	
	/**
	 * Represents the current version number of ProFiDo.
	 */
	public static String PROFIDO_VERSION = "1.3";
	
	/**
	 * Represents the year in which this ProFiDo version was created.
	 */
	public static String PROFIDO_VERSION_YEAR = "2014";
	
	/**
	 * Contains the name of this software.
	 */
	public static String NAME = "ProFiDo";

	/**
	 * The panel used to display the actual information about ProFiDo.
	 */
	private JEditorPane contentPanel;
	
	/**
	 * JPanel that contains all buttons the frame will display.
	 */
	private JPanel buttonPanel;

	/**
	 * The ok button used to close this panel.
	 */
	private JButton okButton;

	/**
	 * The scroll pane used to enable scroll bars within the about text.
	 */
	protected JScrollPane scrollpane;

	/**
	 * Creates and initilazes the panel according to the passed size.
	 */
	public AboutFrame(int width, int height) {
		this.initialize(width, height);
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	public void initialize(int width, int height) {
		this.setSize(width, height);
		this.getContentPane().add(this.getScrollPane(), BorderLayout.NORTH);
		this.getContentPane().add(this.getButtonPanel(), BorderLayout.SOUTH);
		this.setTitle("About ProFiDo");
	}

	/**
	 * Creates (if necessary) and returns the contenPanel.
	 */
	protected JComponent getContentPanel() {
		if (this.contentPanel == null) {
			this.contentPanel = new JEditorPane("text/html", this.getMessage());
			this.contentPanel.setEditable(false);
			this.contentPanel.setCaretPosition(0);
		}
		return this.contentPanel;
	}

	/**
	 * Creates (if necessary) and returns the buttonPanel-
	 */
	protected JPanel getButtonPanel() {
		if (this.buttonPanel == null) {
			this.buttonPanel = new JPanel();
			this.buttonPanel.add(this.getOkButton(this), BorderLayout.CENTER);
		}
		return this.buttonPanel;
	}

	/**
	 * Creates (if necessary) and returns the okButton-
	 */
	protected JButton getOkButton(final JFrame frame) {
		if (this.okButton == null) {
			this.okButton = new JButton("OK");
			this.okButton.setPreferredSize(new Dimension(100, 50));
			this.okButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					frame.setVisible(false);
				}
			});
		}
		return this.okButton;
	}

	/**
	 * Reads and returns the content of "aboutFile.txt" in order to display it
	 * in the about window.
	 */
	protected String getMessage() {
		String message = "";
		InputStream is = null;
		String path = "";
		try {
			path = "/aboutFile.txt";
			is = AboutFrame.class.getResourceAsStream(path);
			BufferedReader bufferedReader = new BufferedReader(
					new InputStreamReader(is));
			while (bufferedReader.ready()) {
				message += bufferedReader.readLine();
			}

			// Dispose all the resources after using them.
			is.close();
			bufferedReader.close();
		} catch (IOException e) {
		}

		message = message.replace("VERSION", AboutFrame.PROFIDO_VERSION);
		message = message.replace("YEAR", AboutFrame.PROFIDO_VERSION_YEAR);
		return message;
	}

	/**
	 * This method creates the scrollpane.
	 */
	public JScrollPane getScrollPane() {
		if (this.scrollpane == null) {
			this.scrollpane = new JScrollPane(this.getContentPanel());
			this.scrollpane.setPreferredSize(new Dimension(700, 600));
		}
		return this.scrollpane;
	}

}
