/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/**
 * This class is opens and display the image file passed to
 * its constructor.
 */
package gui;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import core.TypedFile;

/**
 * This class defines the properties of the "DisplayImage" panel. This panel is
 * used when we want to display images within the gui.
 */
public class DisplayImagePanel extends DisplayPanel {
	private static final long serialVersionUID = 1L;
	
	/**
	 * Reference to the image that should be displayed.
	 */
	private BufferedImage imageToShow;

	/**
	 * Creates the canvas and opens the image.
	 */
	DisplayImagePanel(TypedFile inputFile) {
		try {
			this.imageToShow = ImageIO.read(inputFile);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method paints the given image(imageToShow)
	 */
	@Override
	public void paint(Graphics g) {
		if (this.imageToShow != null) {
			g.drawImage(this.imageToShow, 0, 0, this);
		}
	}

	/**
	 * Returns the preferred dimensions of the canvas.
	 */
	@Override
	public Dimension getPreferredSize() {
		if (this.imageToShow != null)
			return new Dimension(this.imageToShow.getWidth(),
					this.imageToShow.getHeight());
		else
			return new Dimension(200, 200);
	}

	/**
	 * Returns the minimum dimensions of the canvas.
	 */
	@Override
	public Dimension getMinimumSize() {
		return this.getPreferredSize();
	}

	/**
	 * This method saves the displayed Image as a file. The parameter
	 * "targetFile" specifies the name of the file and the location where it
	 * should be saved.
	 */
	@Override
	public void saveContentToFile(File targetFile) throws IOException {
		ImageIO.write(this.imageToShow, "png", targetFile);

	}
}
