/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

/**
 * This class is opens and display the image file passed to
 * its constructor.
 */
package gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JEditorPane;
import javax.swing.JScrollPane;

import core.TypedFile;

/**
 * This class defines the properties of the "DisplayText" panel. This panel is
 * used when we want to display text within the gui.
 */
public class DisplayTextPanel extends DisplayPanel {
	private static final long serialVersionUID = 1L;
	
	/**
	 * JScrollPane used to be able to scroll throw the displayed text.
	 */
	private JScrollPane scrollPane;
	
	/**
	 * JEditorPane used to display the text.
	 */
	private JEditorPane editorPane;
	
	/**
	 * Represents the text that should be viewed.
	 */
	private String textToView;
	
	/**
	 * Restricts the rows that are displayed.
	 */
	private static int MAX_LINES_TO_READ = 1000;

	/**
	 * Creates the canvas and opens the image.
	 */
	public DisplayTextPanel(TypedFile inputFile) {
		super();
		this.setLayout(new BorderLayout());

		// Load content
		this.textToView = "";
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader(
					inputFile));
			int curLine = 1;
			while (bufferedReader.ready()) {
				this.textToView = this.textToView + bufferedReader.readLine()
						+ "\n";
				if (curLine > DisplayTextPanel.MAX_LINES_TO_READ) {
					this.textToView = this.textToView + "\n"
							+ "---Warning---\nOnly the first "
							+ DisplayTextPanel.MAX_LINES_TO_READ
							+ " lines are shown here.\n";
					this.textToView = this.textToView + inputFile.getName();
					break;
				}
				curLine++;
			}
			bufferedReader.close();
		} catch (IOException e) {
		}

		this.add(this.getScrollPane(this.textToView), BorderLayout.CENTER);
		this.setPreferredSize(new Dimension(500, 500));
	}

	/**
	 * Creates (if necessary) and returns the editorPane.
	 */
	protected JEditorPane getEditorPane(String content) {
		if (this.editorPane == null) {
			this.editorPane = new JEditorPane("text/plain", content);
			this.editorPane.setEditable(false);
		}
		return this.editorPane;
	}

	/**
	 * This method returns the ScrollPane if it already exists otherwise it
	 * creates a ScrollPane with the given parameters
	 */
	public JScrollPane getScrollPane(String content) {
		if (this.scrollPane == null) {
			this.scrollPane = new JScrollPane(this.getEditorPane(content));
		}
		return this.scrollPane;
	}

	/**
	 * This method saves the displayed text as a file. The parameter
	 * "targetFile" specifies the name of the file and the location where it
	 * should be saved.
	 */
	@Override
	public void saveContentToFile(File targetFile) throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(targetFile));
		writer.write(this.textToView);
		writer.close();
	}
}
