/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.ProFiDoEditorPaneKeyListener;
import gui.helper.ProfidoDocument;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import core.graph.GraphNode;
import core.history.ChangeNodeCommentHistoryObject;
import core.history.ChangeNodeCommentVisibiltyHistoryObject;
import core.history.History;

/**
 * This class defines the properties of the "About ProFiDo" frame.
 * 
 * It creates the frame which is shown after a click on the "About ProFiDo"
 * item.
 */
public class EditCommentFrame extends JFrame {
	// TODO Improve design, use ShowTextPanel (or create subclass)

	private static final long serialVersionUID = 1L;

	private JPanel buttonPanel;

	/**
	 * The ok button used to close this panel.
	 */
	private JButton okButton;

	/**
	 * The scroll pane used to enable scroll bars within the about text.
	 */
	protected JScrollPane scrollpane;

	/**
	 * Editor pane used to show and edit the comment.
	 */
	private JEditorPane editorPane;

	/**
	 * Button used to close the EditCommentFrame. All changes get lost.
	 */
	private JButton cancelButton;

	/**
	 * Reference to the GraphNode that is edited.
	 */
	private GraphNode nodeToEdit;

	/**
	 * Reference to the History.
	 */
	private History history;

	/**
	 * CheckBox used to change the visibility of the comment.
	 */
	private JCheckBox visibleCheckBox;

	/**
	 * Creates and initilazes the panel according to the passed size.
	 */
	public EditCommentFrame(int width, int height, GraphNode nodeToEdit,
			History history, JPanel parent) {
		this.nodeToEdit = nodeToEdit;
		this.history = history;
		// Save current comment
		this.initialize(width, height);
		this.setTitle("Edit Comment - " + nodeToEdit.getName());
		this.getEditorPane().setText(nodeToEdit.getComment());
		this.setLocationRelativeTo(parent);
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	public void initialize(int width, int height) {
		this.setSize(width, height);
		this.getContentPane().add(this.getScrollPane(), BorderLayout.CENTER);
		this.getContentPane().add(this.getButtonPanel(), BorderLayout.SOUTH);
	}

	/*
	 * protected JPanel getContentPanel() { if(contentPanel == null){
	 * contentPanel = new JPanel("text/html", "");
	 * contentPanel.setEditable(true); contentPanel.setCaretPosition(0); }
	 * return contentPanel; }
	 */

	/**
	 * Creates (if necessary) and returns the contenPanel.
	 */
	protected JEditorPane getEditorPane() {
		if (this.editorPane == null) {
			this.editorPane = new JEditorPane("text/plain", "");
			this.editorPane.setEditable(true);
			this.editorPane.setDocument(new ProfidoDocument());
			this.editorPane.addKeyListener(new ProFiDoEditorPaneKeyListener(
					this.editorPane));
			this.editorPane.setCaretPosition(0);
		}
		return this.editorPane;
	}

	/**
	 * Creates (if necessary) and returns the buttonPanel-
	 */
	protected JPanel getButtonPanel() {
		if (this.buttonPanel == null) {
			this.buttonPanel = new JPanel();
			this.buttonPanel.add(this.getOkButton(this),
					BorderLayout.LINE_START);
			this.buttonPanel.add(this.getCancelButton(this),
					BorderLayout.CENTER);
			this.buttonPanel.add(this.getVisibleCheckBox(),
					BorderLayout.LINE_END);
		}
		return this.buttonPanel;
	}

	/**
	 * This method creates the visibleCheckBox.
	 */
	private JCheckBox getVisibleCheckBox() {
		if (this.visibleCheckBox == null) {
			this.visibleCheckBox = new JCheckBox();
			this.visibleCheckBox.setText("Visible");
			this.visibleCheckBox
					.setSelected(this.nodeToEdit.isCommentVisible());
		}
		return this.visibleCheckBox;
	}

	/**
	 * Creates (if necessary) and returns the okButton-
	 */
	protected JButton getOkButton(final JFrame frame) {
		if (this.okButton == null) {
			this.okButton = new JButton("OK");
			// okButton.setPreferredSize(new Dimension(100, 50));
			this.okButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					if (!EditCommentFrame.this
							.getEditorPane()
							.getText()
							.equals(EditCommentFrame.this.nodeToEdit
									.getComment())) {
						EditCommentFrame.this.history
								.addNewAction(new ChangeNodeCommentHistoryObject(
										EditCommentFrame.this.nodeToEdit,
										EditCommentFrame.this.getEditorPane()
												.getText()));
						EditCommentFrame.this.history.performLastAction();
					}
					if (!EditCommentFrame.this.getVisibleCheckBox()
							.isSelected() == EditCommentFrame.this.nodeToEdit
							.isCommentVisible()) {
						EditCommentFrame.this.history
								.addNewAction(new ChangeNodeCommentVisibiltyHistoryObject(
										EditCommentFrame.this.nodeToEdit,
										EditCommentFrame.this
												.getVisibleCheckBox()
												.isSelected()));
						EditCommentFrame.this.history.performLastAction();
					}
					frame.setVisible(false);
				}

			});
		}
		return this.okButton;
	}

	/**
	 * Creates (if necessary) and returns the okButton-
	 */
	protected JButton getCancelButton(final JFrame frame) {
		if (this.cancelButton == null) {
			this.cancelButton = new JButton("Cancel");
			// okButton.setPreferredSize(new Dimension(100, 50));
			this.cancelButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					frame.setVisible(false);
				}
			});
		}
		return this.cancelButton;
	}

	/**
	 * This method creates the scrollpane.
	 */
	public JScrollPane getScrollPane() {
		if (this.scrollpane == null) {
			this.scrollpane = new JScrollPane(this.getEditorPane());
			// scrollpane.setPreferredSize(new Dimension(200, 200));
		}
		return this.scrollpane;
	}

}
