/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.InstantViewManager;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JPanel;

import core.graph.JobNode;

/**
 * This class defines the properties of the "Edit Job" frame. It consists only
 * of one EditJobPanel.
 */

public class EditJobFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	/**
	 * Reference to the content panel.
	 */
	private JPanel contentPane = null;

	/**
	 * Job node which this frame is assigned to.
	 */
	private JobNode myJob;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param instantViewManager
	 */
	public EditJobFrame(final JobNode myJob, final WorkflowPanel workflowPanel,
			InstantViewManager instantViewManager) {
		super();
		this.myJob = myJob;
		this.workflowPanel = workflowPanel;
		myJob.setEditDialog(this);
		this.setContentPane(this.getJContentPane(myJob, this.workflowPanel,
				instantViewManager));
		if (instantViewManager == null) {
			this.setTitle("Edit Job - " + myJob.getName());
		} else {
			this.setTitle("Setup edge view - " + myJob.getName());
		}

		this.addWindowListener(new WindowListener() {
			@Override
			public void windowClosing(WindowEvent e) {
				// System.out.println(getTitle() + " closing.");
				myJob.setEditDialog(null);
				EditJobFrame.this.repaint();
			}

			@Override
			public void windowActivated(WindowEvent e) {
				myJob.setCurrentlyEdited(true);
				workflowPanel.repaint();
			}

			@Override
			public void windowClosed(WindowEvent e) {
			}

			@Override
			public void windowDeactivated(WindowEvent e) {
				myJob.setCurrentlyEdited(false);
				workflowPanel.repaint();
			}

			@Override
			public void windowDeiconified(WindowEvent e) {
			}

			@Override
			public void windowIconified(WindowEvent e) {
			}

			@Override
			public void windowOpened(WindowEvent e) {
			}
		});
	}

	/**
	 * This method creates the contentPane for a given JobNode, WorkflowPanel
	 * and a InstantViewManager. This contentPane is the actual content.
	 */
	private JPanel getJContentPane(JobNode myJob, WorkflowPanel workflowPanel,
			InstantViewManager instantViewManager) {
		if (this.contentPane == null) {
			this.contentPane = new EditJobPanel(myJob, workflowPanel, this,
					instantViewManager);
			this.setSize(500,
					((EditJobPanel) this.contentPane).getDesiredHeight() + 40);
			this.contentPane.setLayout(null);
		}
		return this.contentPane;
	}

	/**
	 * Informs this frame that any interaction the user wants to have with it is
	 * finished.
	 */
	public void finished() {
		this.myJob.setEditDialog(null);
		this.dispose();
	}

}
