/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.event.MouseInputAdapter;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

/**
 * This class defines a panel displaying a text with additional information.
 */

public class HyperlinkPanel extends JPanel {
	// TODO Improve desgin

	private static final long serialVersionUID = 1L;
	private JPanel headingPanel = new JPanel();
	private JPanel closingPanel = new JPanel();
	private final static String LINK_ATTRIBUTE = "linkact";
	private JTextPane textPane;
	private StyledDocument doc;

	/**
	 * Creates and initializes the panel.
	 */
	public HyperlinkPanel(String url) {
		try {
			Dimension screenDimension = Toolkit.getDefaultToolkit()
					.getScreenSize();
			Rectangle containerDimension = this.getBounds();
			int toolXPosition = (screenDimension.width - containerDimension.width) / 10;
			int toolYPosition = (screenDimension.height - containerDimension.height) / 15;

			this.setLocation(toolXPosition, toolYPosition);
			this.add(BorderLayout.NORTH, this.headingPanel);
			this.add(BorderLayout.SOUTH, this.closingPanel);

			JScrollPane scrollableTextPane;
			this.textPane = new JTextPane();
			// for detecting clicks
			this.textPane.addMouseListener(new TextClickListener());
			// for detecting motion
			this.textPane.addMouseMotionListener(new TextMotionListener());
			this.textPane.setEditable(false);
			scrollableTextPane = new JScrollPane(this.textPane);
			this.add(BorderLayout.CENTER, scrollableTextPane);
			this.setVisible(true);
			this.textPane.setText("");
			this.doc = this.textPane.getStyledDocument();
			Style def = StyleContext.getDefaultStyleContext().getStyle(
					StyleContext.DEFAULT_STYLE);

			Style regularBlue = this.doc.addStyle("regularBlue", def);
			StyleConstants.setForeground(regularBlue, Color.BLUE);
			StyleConstants.setUnderline(regularBlue, true);
			regularBlue.addAttribute(HyperlinkPanel.LINK_ATTRIBUTE,
					new URLLinkAction(url));

			Style bold = this.doc.addStyle("bold", def);
			StyleConstants.setBold(bold, true);
			StyleConstants.setForeground(bold, Color.GRAY);

			this.doc.insertString(this.doc.getLength(),
					"\nProfido Help. Click the link below\n\n", bold);
			this.doc.insertString(this.doc.getLength(), "\n", bold);
			this.doc.insertString(this.doc.getLength(), url, regularBlue);
			this.doc.insertString(this.doc.getLength(), "\n\n\n", bold);
			this.textPane.setCaretPosition(0);
		} catch (Exception e) {
		}
	}

	/**
	 * Action listener to start the browser when the hyperlink is clicked.
	 */
	private class TextClickListener extends MouseAdapter {
		@Override
		public void mouseClicked(MouseEvent e) {
			try {
				Element elem = HyperlinkPanel.this.doc
						.getCharacterElement(HyperlinkPanel.this.textPane
								.viewToModel(e.getPoint()));
				AttributeSet as = elem.getAttributes();
				URLLinkAction fla = (URLLinkAction) as
						.getAttribute(HyperlinkPanel.LINK_ATTRIBUTE);
				if (fla != null) {
					fla.execute();
				}
			} catch (Exception x) {
				x.printStackTrace();
			}
		}
	}

	private class TextMotionListener extends MouseInputAdapter {
		@Override
		public void mouseMoved(MouseEvent e) {
			Element elem = HyperlinkPanel.this.doc
					.getCharacterElement(HyperlinkPanel.this.textPane
							.viewToModel(e.getPoint()));
			AttributeSet as = elem.getAttributes();
			if (StyleConstants.isUnderline(as)) {
				HyperlinkPanel.this.textPane.setCursor(new Cursor(
						Cursor.HAND_CURSOR));
			} else {
				HyperlinkPanel.this.textPane.setCursor(new Cursor(
						Cursor.DEFAULT_CURSOR));
			}
		}
	}

	/**
	 * Class implementing an action which starts a browser according to the
	 * supplied information.
	 */
	private class URLLinkAction extends AbstractAction {
		private String url;

		URLLinkAction(String bac) {
			this.url = bac;
		}

		/**
		 * This method is called after a click on the link and causes a visit on
		 * the web site.
		 */

		protected void execute() {
			try {
				String osName = System.getProperty("os.name").toLowerCase();
				Runtime rt = Runtime.getRuntime();
				if (osName.indexOf("win") >= 0) {
					rt.exec("rundll32 url.dll,FileProtocolHandler " + this.url);
				} else if (osName.indexOf("mac") >= 0) {
					rt.exec("open " + this.url);
				}

				else if (osName.indexOf("ix") >= 0 || osName.indexOf("ux") >= 0
						|| osName.indexOf("sun") >= 0) {
					String[] browsers = { "epiphany", "firefox", "mozilla",
							"konqueror", "netscape", "opera", "links", "lynx" };

					// Build a command string which looks like
					// "browser1 "url" || browser2 "url" ||..."
					StringBuffer cmd = new StringBuffer();
					for (int i = 0; i < browsers.length; i++) {
						cmd.append((i == 0 ? "" : " || ") + browsers[i] + " \""
								+ this.url + "\" ");
					}

					rt.exec(new String[] { "sh", "-c", cmd.toString() });
				}
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			this.execute();
		}
	}
}
