/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import core.TypedFile;

/**
 * This class defines the properties of the "InstantView" frame. This frame is
 * used when we want to display files within the gui. The actual content is
 * created by the InstantViewPanel class.
 */
public class InstantViewFrame extends JFrame {
	private static final long serialVersionUID = 1L;

	/**
	 * Reference to a InstantViewPanel object called contentPane which
	 * represents the actual content.
	 */
	private InstantViewPanel contentPane;

	/**
	 * Create the frame.
	 */
	public InstantViewFrame(TypedFile inputFile, String type,
			WorkflowPanel workflowPanel) {
		super();
		this.setTitle("Instant View - " + inputFile.getName());
		if (inputFile.exists()) {
			this.setContentPane(this.getJContentPane(inputFile, type,
					workflowPanel));
			this.pack();
			this.setVisible(true);
		} else {
			JOptionPane
					.showMessageDialog(
							workflowPanel,
							"The file "
									+ inputFile.getName()
									+ " does not exist.\nPlease export and execute the workflow fist.",
							"Error", JOptionPane.ERROR_MESSAGE);
			this.setVisible(false);
		}

		this.addWindowListener(new WindowListener() {
			@Override
			public void windowClosing(WindowEvent e) {
				// contentPane.panelClosing();
			}

			@Override
			public void windowActivated(WindowEvent arg0) {
			}

			@Override
			public void windowClosed(WindowEvent arg0) {
			}

			@Override
			public void windowDeactivated(WindowEvent arg0) {
			}

			@Override
			public void windowDeiconified(WindowEvent arg0) {
			}

			@Override
			public void windowIconified(WindowEvent arg0) {
			}

			@Override
			public void windowOpened(WindowEvent arg0) {
			}
		});
	}

	/**
	 * This method returns the contentPane if it already exists otherwise it
	 * creates a new contentPane with the given parameters
	 */
	private JPanel getJContentPane(TypedFile inputFile, String type,
			WorkflowPanel workflowPanel) {
		if (this.contentPane == null) {
			this.contentPane = new InstantViewPanel(inputFile, type,
					workflowPanel);
		}
		return this.contentPane;
	}

	/**
	 * This method is called when the frame is no longer needed
	 */
	public void finished() {
		this.dispose();
	}
}
