/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.DialogManager;

import java.awt.BorderLayout;
import java.awt.Component;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JPanel;

import core.TypedFile;

/**
 * This class defines the properties of the "InstantView" panel. This panel is
 * used when we want to display files within the gui. Mostly used to display
 * images and text. The actual content is created by either the DisplayTextPanal
 * class or the DisplayImagePanal class.
 */
public class InstantViewPanel extends JPanel {
	static final long serialVersionUID = 1L;
	
	/**
	 * Reference to a JButton called saveContentButton.
	 */
	private JButton saveContentButton;
	
	/**
	 * Reference to a displayPanel.
	 */
	private DisplayPanel displayPanel;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;
	
	/**
	 * Reference to a TypedFile called fileName which will be displayed within the GUI.
	 */
	private TypedFile fileName;

	/**
	 * Creates the panel.
	 */
	public InstantViewPanel(TypedFile inputFile, String type,
			WorkflowPanel workflowPanel) {
		super();
		this.initialize(inputFile, type);
		this.fileName = inputFile;
		this.workflowPanel = workflowPanel;
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize(TypedFile inputFile, String type) {
		// BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(new BorderLayout());

		this.add(this.getViewerPanel(inputFile, type), BorderLayout.CENTER);
		this.add(this.getSaveContentButton(), BorderLayout.PAGE_END);
	}

	/**
	 * This method returns the displayPanel if it already exists. Otherwise it
	 * creates a new DisplayTextPanel or DisplayImagePanel depends on which type
	 * is passed to the method.
	 */
	private DisplayPanel getViewerPanel(TypedFile inputFile, String type) {
		if (this.displayPanel == null) {
			if (type.equals("text")) {
				this.displayPanel = new DisplayTextPanel(inputFile);
			} else if (type.equals("image")) {
				this.displayPanel = new DisplayImagePanel(inputFile);
			}
		}

		return this.displayPanel;
	}

	/**
	 * This method creates the SaveContentButton. The Button appears in the
	 * ViewerPanel and allows the user to save the displayed image or text.
	 */
	private JButton getSaveContentButton() {
		if (this.saveContentButton == null) {
			this.saveContentButton = new JButton("Save to file...");
			this.saveContentButton.setFocusable(false);
			this.saveContentButton.setAlignmentX(Component.CENTER_ALIGNMENT);
			this.saveContentButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							String proposedFileName = InstantViewPanel.this.fileName
									.getName();
							File targetFile = DialogManager.showSaveDialog(
									InstantViewPanel.this.workflowPanel
											.getCurrentWorkingDirectory(),
									proposedFileName, ".*",
									InstantViewPanel.this.workflowPanel, true);
							try {
								if (targetFile != null) {
									InstantViewPanel.this.displayPanel
											.saveContentToFile(targetFile);
								}
							} catch (IOException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}
						}
					});
		}
		return this.saveContentButton;
	}

}
