/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.DialogManager;
import gui.helper.InstantViewManager;

import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.WindowConstants;

import core.storage.ConfigurationXMLReader;

/**
 * This class defines the properties of main frame of the gui.
 * 
 * The MainWindowPanel consists of the MainWindowPanel and the applications menu
 * bar with all its items e.g. File, Batch, Edit... Each menu item has an action
 * listener which are defined in this class
 */

public class MainWindowFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	/**
	 * Reference to this frames main panel.
	 */
	private MainWindowPanel mainWindowPanel = null;

	/**
	 * Reference to the panel showing the workflow.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the configuration xml reader.
	 */
	private ConfigurationXMLReader configurationXMLReader;

	/**
	 * Main menu bar.
	 */
	private JMenuBar MainMenuBar = null;

	/**
	 * Reference to the this MainWindowFrame.
	 */
	private MainWindowFrame me;

	/**
	 * Reference to the batchMenu
	 */
	private JMenu batchMenu = null;

	/**
	 * Reference to a JMenu called "AddJobMenu" used to add Jobs to the
	 * WorkFlow.
	 */
	private JMenu AddJobMenu;

	/**
	 * Reference to a JMenu called "helpMenu" that is used to display help
	 * options for the user.
	 */
	private JMenu helpMenu;

	/**
	 * Menu item to access the help text.
	 */
	private JMenuItem helpContentItem;

	/**
	 * Menu item to access the about frame.
	 */
	private JMenuItem aboutItem;

	/**
	 * Checkbox that is used to give the user the choice whether the Workflow
	 * should be resized or not.
	 */
	private JCheckBoxMenuItem ResizeCheckBoxM = null;

	/**
	 * Reference to a JMenu called "settingsMenu" that is used to access the
	 * ProFiDo settings menu.
	 */
	private JMenu settingsMenu;

	/**
	 * Checkbox that is used to give the user the choice whether the gird is
	 * visible or not.
	 */
	private JCheckBoxMenuItem ShowGridCheckBoxM;

	/**
	 * Reference to a JMenu called "fileMenu" used to access the file menu.
	 */
	private JMenu fileMenu;

	/**
	 * Menu item to load a Workflow/graph. A JFileChooser will appear.
	 */
	private JMenuItem loadGraphMenuItem;

	/**
	 * Menu item to save the current Workflow to the file system.
	 */
	private JMenuItem saveGraphMenuItem;

	/**
	 * Checkbox that is used to give the user the choice to display the edge
	 * information or disable them.
	 */
	private JCheckBoxMenuItem displayEdgeInformationCheckBox;

	/**
	 * Checkbox that is used to give the user the ability to display the node
	 * information or to hide them.
	 */
	private JCheckBoxMenuItem displayNodeInformationCheckBox;

	/**
	 * Reference to a JCheckBoxMenuItem called "alwaysDisplayOrderKeyCheckBox"
	 * that gives the user the ability to enable/disable whether the OrderKeys
	 * should be displayed or not
	 */
	private JCheckBoxMenuItem alwaysDisplayOrderKeyCheckBox;

	/**
	 * Checkbox that is used to give the user the ability to display all node
	 * information or to hide them.
	 */
	private JCheckBoxMenuItem displayAllEdgeInformationCheckBox;

	/**
	 * Checkbox used to give the user the choice to display all edge information
	 * or disable them.
	 */
	private JCheckBoxMenuItem displayAllNodeInformationCheckBox;

	/**
	 * Reference to a JMenu called "debugMenu" used to access the debug menu.
	 */
	private JMenu debugMenu;

	/**
	 * Menu item to change the name of the current Workflow.
	 */
	private JMenuItem editNameMenuItem;

	/**
	 * Reference to a JMenu called "editMenu" used to access the edit menu that
	 * contains options to access the History.
	 */
	private JMenu editMenu;

	/**
	 * Menu item to redo a History action.
	 */
	private JMenuItem redoMenuItem;

	/**
	 * Menu item to undo the last action the user performed.
	 */
	private JMenuItem undoMenuItem;

	/**
	 * Menu Item to create a new output node.
	 */
	private JMenuItem newOutputFileMenuItem;

	/**
	 * Menu item to create a new input node.
	 */
	private JMenuItem newInputFileMenuItem;

	/**
	 * Menu item to export the workflow as a shell script.
	 */
	private JMenuItem exportScriptMenuItem;

	/**
	 * Menu item to set a new output folder.
	 */
	private JMenuItem setOutputFolderMenuItem;

	/**
	 * Menu item to create a new workflow.
	 */
	private JMenuItem newWorkflowItem;

	/**
	 * Menu item to show the History.
	 */
	private JMenuItem showHistoryMenuItem;

	/**
	 * Menu item to close ProFiDo.
	 */
	private JMenuItem quitMenuItem;

	/**
	 * Menu item to clear the History.
	 */
	private JMenuItem clearHistoryMenuItem;

	/**
	 * Menu item to export the current Workflow as a screenshot.
	 */
	private JMenuItem exportScreenShotMenuItem;

	/**
	 * Determined whether the xml schema check should be performed or not.
	 */
	private boolean checkSchema;

	/**
	 * Menu item to export the current workflow as a script that includes the
	 * report.
	 */
	private JMenuItem exportReportMenuItem;

	/**
	 * Creates and initializes the panel.
	 */
	public MainWindowFrame(ConfigurationXMLReader reader) {
		super();
		this.me = this;
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.configurationXMLReader = reader;

		this.initialize();
		this.mainWindowPanel.updatePositions();

	}

	/**
	 * This method close the window and the entire application after verifying
	 * if the graph was saved.
	 * 
	 * @param performCheckIfSaved
	 */
	public void exitApplication(boolean performCheckIfSaved) {
		boolean exitApplication = (!performCheckIfSaved) || 
								  (performCheckIfSaved && ensureWorkflowIsSaved("You are about to close the application without saving the current workflow.\nWould you like to save the current workflow before exiting?"));

		if (exitApplication) {
			System.exit(0);
		}

	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize() {
		this.setSize(600, 600);
		this.setContentPane(this.getJContentPane());
		this.setJMenuBar(this.getMainMenuBar());
		this.updateTitle("");
		this.addWindowListener(new WindowListener() {
			@Override
			public void windowOpened(WindowEvent e) {
			}

			@Override
			public void windowIconified(WindowEvent e) {
			}

			@Override
			public void windowDeiconified(WindowEvent e) {
			}

			@Override
			public void windowDeactivated(WindowEvent e) {
			}

			@Override
			public void windowClosing(WindowEvent e) {
				MainWindowFrame.this.exitApplication(true);
			}

			@Override
			public void windowClosed(WindowEvent e) {
			}

			@Override
			public void windowActivated(WindowEvent e) {
			}
		});
		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {

			}
		});
	}

	/**
	 * This method creates the mainWindowPanel (if necessary) and returns it.
	 * 
	 * @return mainWindowPanel that represents the content.
	 */
	private JPanel getJContentPane() {
		if (this.mainWindowPanel == null) {
			this.mainWindowPanel = new MainWindowPanel(
					this.configurationXMLReader, this);
			this.workflowPanel = this.mainWindowPanel.getWorkflowPanel(
					this.configurationXMLReader, this);
		}
		return this.mainWindowPanel;
	}

	/**
	 * This method creates the MainMenuBar (if necessary) and returns it.
	 */
	private JMenuBar getMainMenuBar() {
		if (this.MainMenuBar == null) {
			this.MainMenuBar = new JMenuBar();
			this.MainMenuBar.add(this.getFileMenu());
			this.MainMenuBar.add(this.getBatchMenu());
			this.MainMenuBar.add(this.getEditMenu());
			this.MainMenuBar.add(this.getSettingsMenu());
			this.MainMenuBar.add(this.getHelpMenu());
		}
		return this.MainMenuBar;
	}

	/**
	 * This method creates the editMenu (if necessary) and returns it.
	 */
	private JMenu getEditMenu() {
		if (this.editMenu == null) {
			this.editMenu = new JMenu();
			this.editMenu.setText("Edit");
			this.editMenu.add(this.getUndoMenuItem());
			this.editMenu.add(this.getRedoMenuItem());
			this.editMenu.addSeparator();
			this.editMenu.add(this.getShowHistoryMenuItem());
			this.editMenu.add(this.getClearHistoryMenuItem());
		}
		return this.editMenu;
	}

	/**
	 * This method creates the batchMenu (if necessary) and returns it.
	 */
	private JMenu getBatchMenu() {
		if (this.batchMenu == null) {
			this.batchMenu = new JMenu();
			this.batchMenu.setText("Workflow");
			this.batchMenu.add(this.getAddJobMenu());
			this.batchMenu.add(this.getNewOutputFileMenuItem());
			this.batchMenu.add(this.getNewInputFileMenuItem());
			this.batchMenu.add(this.getEditNameMenuItem());
			this.batchMenu.add(this.getSetOutputFolderMenuItem());
			this.batchMenu.addSeparator();
			this.batchMenu.add(this.getExportScriptMenuItem());
			this.batchMenu.add(this.getExportReportMenuItem());
			this.batchMenu.add(this.getExportScreenShotMenuItem());
		}
		return this.batchMenu;
	}

	/**
	 * This method creates the fileMenu (if necessary) and returns it.
	 */
	private JMenu getFileMenu() {
		if (this.fileMenu == null) {
			this.fileMenu = new JMenu();
			this.fileMenu.setText("File");

			this.fileMenu.add(this.getNewWorkflowMenuItem());
			this.fileMenu.addSeparator();
			this.fileMenu.add(this.getLoadGraphMenuItem());
			this.fileMenu.add(this.getSaveGraphMenuItem());
			this.fileMenu.addSeparator();
			this.fileMenu.add(this.getQuitMenuItem());
		}
		return this.fileMenu;
	}

	/**
	 * This method creates the redoMenuItem (if necessary) and returns it.
	 */
	private JMenuItem getRedoMenuItem() {
		if (this.redoMenuItem == null) {
			this.redoMenuItem = new JMenuItem();
			this.redoMenuItem.setText("Redo");
			this.redoMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.redo();
							MainWindowFrame.this.workflowPanel.repaint();
						}
					});
		}
		return this.redoMenuItem;
	}

	/**
	 * This method creates the undoMenuItem (if necessary) and returns it.
	 */
	public JMenuItem getUndoMenuItem() {
		if (this.undoMenuItem == null) {
			this.undoMenuItem = new JMenuItem();
			this.undoMenuItem.setText("Undo");
			this.undoMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.undo();
							MainWindowFrame.this.workflowPanel.repaint();
						}
					});
		}
		return this.undoMenuItem;
	}

	/**
	 * This method creates the clearHistoryMenuItem (if necessary) and returns
	 * it.
	 */
	private JMenuItem getClearHistoryMenuItem() {
		if (this.clearHistoryMenuItem == null) {
			this.clearHistoryMenuItem = new JMenuItem();
			this.clearHistoryMenuItem.setText("Clear History");
			this.clearHistoryMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.clearHistory();
						}
					});
		}
		return this.clearHistoryMenuItem;
	}

	/**
	 * This method creates the showHistoryMenuItem (if necessary) and returns
	 * it.
	 */
	private JMenuItem getShowHistoryMenuItem() {
		if (this.showHistoryMenuItem == null) {
			this.showHistoryMenuItem = new JMenuItem();
			this.showHistoryMenuItem.setText("Show History");
			this.showHistoryMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.showHistory();
						}
					});
		}
		return this.showHistoryMenuItem;
	}

	/**
	 * This method creates the exportScriptMenuItem (if necessary) and returns
	 * it.
	 */
	private JMenuItem getExportScriptMenuItem() {
		if (this.exportScriptMenuItem == null) {
			this.exportScriptMenuItem = new JMenuItem();
			this.exportScriptMenuItem.setText("Export Workflow as Script");
			this.exportScriptMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							if (MainWindowFrame.this.workflowPanel.check() == true) {
								File targetName = DialogManager.showSaveDialog(
										MainWindowFrame.this.workflowPanel
												.getCurrentWorkingDirectory(),
										MainWindowFrame.this.mainWindowPanel
												.getWorkflowName() + ".sh",
										".sh", MainWindowFrame.this.me, true);
								if (targetName != null) {
									MainWindowFrame.this.workflowPanel
											.exportWorkflowScript(
													targetName.getPath(), false);
								}
							}
						}
					});
		}
		return this.exportScriptMenuItem;
	}

	/**
	 * This method creates the exportReportMenuItem (if necessary) and returns
	 * it.
	 */
	private JMenuItem getExportReportMenuItem() {
		if (this.exportReportMenuItem == null) {
			this.exportReportMenuItem = new JMenuItem();
			this.exportReportMenuItem
					.setText("Export Workflow as Script with Report");
			this.exportReportMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							if (MainWindowFrame.this.workflowPanel.check() == true) {
								File targetName = DialogManager.showSaveDialog(
										MainWindowFrame.this.workflowPanel
												.getCurrentWorkingDirectory(),
										MainWindowFrame.this.mainWindowPanel
												.getWorkflowName() + ".sh",
										".sh", MainWindowFrame.this.me, true);
								if (targetName != null) {
									MainWindowFrame.this.workflowPanel
											.exportWorkflowScript(
													targetName.getPath(), true);
								}
							}
						}
					});
		}
		return this.exportReportMenuItem;
	}

	/**
	 * This method creates the exportScreenShotMenuItem (if necessary) and
	 * returns it.
	 */
	private JMenuItem getExportScreenShotMenuItem() {
		if (this.exportScreenShotMenuItem == null) {
			this.exportScreenShotMenuItem = new JMenuItem();
			this.exportScreenShotMenuItem.setText("Save Screenshot");
			this.exportScreenShotMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {

							File targetName = DialogManager.showSaveDialog(
									MainWindowFrame.this.workflowPanel
											.getCurrentWorkingDirectory(),
									MainWindowFrame.this.mainWindowPanel
											.getWorkflowName() + ".png",
									".png", MainWindowFrame.this.me, true);
							if (targetName != null) {
								MainWindowFrame.this.workflowPanel
										.saveScreenshot(targetName.getPath());
							}
						}
					});
		}
		return this.exportScreenShotMenuItem;
	}

	/**
	 * This method creates the loadGraphMenuItem (if necessary) and returns it.
	 */
	private JMenuItem getLoadGraphMenuItem() {
		if (this.loadGraphMenuItem == null) {
			this.loadGraphMenuItem = new JMenuItem();
			this.loadGraphMenuItem.setText("Load Workflow");
			this.loadGraphMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							if (ensureWorkflowIsSaved("You are about to load a workflow without saving the current workflow.\nWould you like to save the current workflow now?")) {

				
								File targetName = DialogManager.showOpenDialog(
										MainWindowFrame.this.workflowPanel
												.getCurrentWorkingDirectory(),
										MainWindowFrame.this.mainWindowPanel
												.getWorkflowName(), ".xml",
										MainWindowFrame.this.me, false);
								if (targetName != null) {
									try {
										MainWindowFrame.this.me.openGraph(
												targetName.getPath(), true);
									} catch (Exception e1) {
										e1.printStackTrace();
									}
								}	
							}
							
							
						}
					});
		}
		return this.loadGraphMenuItem;
	}

	/**
	 * This method creates the saveGraphMenuItem (if necessary) and returns it.
	 */
	private JMenuItem getSaveGraphMenuItem() {
		if (this.saveGraphMenuItem == null) {
			this.saveGraphMenuItem = new JMenuItem();
			this.saveGraphMenuItem.setText("Save Workflow");
			this.saveGraphMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							File targetName = DialogManager.showSaveDialog(
									MainWindowFrame.this.workflowPanel
											.getCurrentWorkingDirectory(),
									MainWindowFrame.this.mainWindowPanel
											.getWorkflowName(), ".xml",
									MainWindowFrame.this.me, true);
							if (targetName != null) {
								MainWindowFrame.this.workflowPanel
										.saveGraph(targetName.getPath());
							}
						}

					});
		}
		return this.saveGraphMenuItem;
	}

	/**
	 * This method creates the settingsMenu (if necessary) and returns it.
	 */
	private JMenu getSettingsMenu() {
		if (this.settingsMenu == null) {
			this.settingsMenu = new JMenu();
			this.settingsMenu.setText("Settings");
			this.settingsMenu.add(this.getResizeCheckBoxM());
			this.settingsMenu.add(this.getShowGridCheckBoxM());
			this.settingsMenu.add(this.getDisplayEdgeInformationCheckBox());
			this.settingsMenu.add(this.getDisplayNodeInformationCheckBox());
			this.settingsMenu.add(this.getDebugMenu());
		}
		this.workflowPanel.setPrintInformation(
				this.displayEdgeInformationCheckBox.getState(),
				this.displayNodeInformationCheckBox.getState(),
				this.alwaysDisplayOrderKeyCheckBox.getState(),
				this.displayAllEdgeInformationCheckBox.getState(),
				this.displayAllNodeInformationCheckBox.getState());
		return this.settingsMenu;
	}

	/**
	 * This method creates the debugMenu (if necessary) and returns it.
	 */
	private JMenu getDebugMenu() {
		if (this.debugMenu == null) {
			this.debugMenu = new JMenu();
			this.debugMenu.setText("Debug");
			this.debugMenu.add(this.getAlwaysDisplayOrderKeyCheckBox());
			this.debugMenu.add(this.getDisplayAllEdgeInformationCheckBox());
			this.debugMenu.add(this.getDisplayAllNodeInformationCheckBox());
		}
		return this.debugMenu;
	}

	/**
	 * This method creates the AddJobMenu (if necessary) and returns it.
	 */
	private JMenu getAddJobMenu() {
		if (this.AddJobMenu == null) {
			this.AddJobMenu = new JMenu();
			this.AddJobMenu.setText("Add Job");
			this.createAddJobMenu(this.AddJobMenu);
		}
		return this.AddJobMenu;
	}

	/**
	 * This method creates the newInputFileMenu (if necessary) and returns it.
	 */
	private JMenuItem getNewInputFileMenuItem() {
		if (this.newInputFileMenuItem == null) {
			this.newInputFileMenuItem = new JMenuItem();
			this.newInputFileMenuItem.setText("Add Input File");
			this.newInputFileMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.newInputFile();
						}
					});
		}
		return this.newInputFileMenuItem;
	}

	/**
	 * This method creates the quitMenuItem (if necessary) and returns it.
	 */
	private JMenuItem getQuitMenuItem() {
		if (this.quitMenuItem == null) {
			this.quitMenuItem = new JMenuItem();
			this.quitMenuItem.setText("Quit");
			this.quitMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.exitApplication(true);
						}

					});
		}
		return this.quitMenuItem;
	}

	/**
	 * This method creates the newOutputFileMenuItem (if necessary) and returns
	 * it.
	 */
	private JMenuItem getNewOutputFileMenuItem() {
		if (this.newOutputFileMenuItem == null) {
			this.newOutputFileMenuItem = new JMenuItem();
			this.newOutputFileMenuItem.setText("Add Output File");
			this.newOutputFileMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel.newOutputFile();
						}
					});
		}
		return this.newOutputFileMenuItem;
	}

	/**
	 * This method creates the editNameMenuItem (if necessary) and returns it.
	 */
	private JMenuItem getEditNameMenuItem() {
		if (this.editNameMenuItem == null) {
			this.editNameMenuItem = new JMenuItem();
			this.editNameMenuItem.setText("Change Workflow Name");
			this.editNameMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							String newName = InstantViewManager
									.createOptionPane(
											MainWindowFrame.this.workflowPanel
													.getBatchName(),
											"Edit Batch Name",
											"Enter new Name:",
											MainWindowFrame.this.workflowPanel);

							if (newName != null) {
								MainWindowFrame.this.workflowPanel
										.setBatchName(newName);
							}

						}
					});
		}
		return this.editNameMenuItem;
	}

	/**
	 * This method creates the setOutputFolderMenuItem (if necessary) and
	 * returns it.
	 */
	private JMenuItem getSetOutputFolderMenuItem() {
		if (this.setOutputFolderMenuItem == null) {
			this.setOutputFolderMenuItem = new JMenuItem();
			this.setOutputFolderMenuItem.setText("Set Working Directory");
			this.setOutputFolderMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							File newPath = DialogManager.showSelectFolderDialog(
									MainWindowFrame.this.workflowPanel
											.getCurrentWorkingDirectory(),
									MainWindowFrame.this.me);
							if (newPath != null) {
								MainWindowFrame.this.workflowPanel
										.setWorkingDirectory(newPath.getPath());
							}
						}
					});
		}
		return this.setOutputFolderMenuItem;
	}

	
	/**
	 * Checks if the workflow is already saved. If not, a dialog promting
	 * the user to do so is showed. 
	 * 
	 * @return If it is save to proceed.
	 */
	private boolean ensureWorkflowIsSaved(String message) {
		boolean result = true;
		
		if (!me.workflowPanel.isGraphSaved()) {
			int dialogResult = JOptionPane
					.showOptionDialog(
							MainWindowFrame.this.me,
							message, 
							"Workflow not saved.",
							JOptionPane.YES_NO_OPTION,
							JOptionPane.WARNING_MESSAGE, null,
							null, null);
			if (dialogResult == JOptionPane.YES_OPTION) {
				// Get file name to save
				File targetName = DialogManager.showSaveDialog(
						MainWindowFrame.this.workflowPanel
								.getCurrentWorkingDirectory(),
						MainWindowFrame.this.mainWindowPanel
								.getWorkflowName(), ".xml",
						MainWindowFrame.this.me, true);
				if (targetName != null) {
					MainWindowFrame.this.workflowPanel
							.saveGraph(targetName.getPath());
				} else {
					// Only create new workflow, if a valid file
					// name was selected
					result = false;
				}
			}

		}
		return result;
	}
	
	/**
	 * This method creates the newWorkflowItem (if necessary) and returns it.
	 */
	private JMenuItem getNewWorkflowMenuItem() {
		if (this.newWorkflowItem == null) {
			this.newWorkflowItem = new JMenuItem();
			this.newWorkflowItem.setText("New Workflow");
			this.newWorkflowItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {						
							boolean createNewWorkflow = true;

							createNewWorkflow = ensureWorkflowIsSaved("You are about to create a new workflow without saving the current workflow.\nWould you like to save the current workflow now?");

							if (createNewWorkflow) {
								String newName = InstantViewManager
										.createOptionPane(
												"workflow",
												"Enter Name",
												"Enter Workflow Name:",
												MainWindowFrame.this.workflowPanel);

								if (newName != null) {
									MainWindowFrame.this.workflowPanel
											.newWorkflow(newName);
								}
							}
						}

					
					});
		}
		return this.newWorkflowItem;
	}

	/**
	 * This method creates the helpMenu (if necessary) and returns it.
	 */
	public JMenu getHelpMenu() {
		if (this.helpMenu == null) {
			this.helpMenu = new JMenu("Help");
			this.helpMenu.add(this.getHelpContentItem());
			this.helpMenu.addSeparator();
			this.helpMenu.add(this.getAboutItem());
		}
		return this.helpMenu;
	}

	/**
	 * This method updates the title of the main window frame. It is called
	 * after changing / setting the batch name.
	 * 
	 * @param newName
	 *            the new name of the batch
	 */
	protected void updateTitle(String newName) {
		if (!newName.equals("")) {
			this.setTitle(AboutFrame.NAME + " - " + newName);
		} else {
			this.setTitle(AboutFrame.NAME);
		}
	}

	/**
	 * This method adds all Jobs specified in the software.xml and adds a menu
	 * item to the AddJobMenu.
	 */
	private void createAddJobMenu(JMenu pAddJobMenu) {
		Vector<String> items = this.configurationXMLReader.getProgrammNames();

		for (int curPos = 0; curPos < items.size(); curPos++) {
			String curProgrammNodeName = items.get(curPos);
			TaggedJMenuItem tmpProgrammMenuItem = new TaggedJMenuItem();
			tmpProgrammMenuItem.setText(curProgrammNodeName);
			tmpProgrammMenuItem.setID(curPos);
			tmpProgrammMenuItem
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.newJob(((TaggedJMenuItem) e.getSource())
											.getID());
						}
					});
			pAddJobMenu.add(tmpProgrammMenuItem);
		}
	}

	/**
	 * This method creates the ResizeCheckBoxM (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getResizeCheckBoxM() {
		if (this.ResizeCheckBoxM == null) {
			this.ResizeCheckBoxM = new JCheckBoxMenuItem();
			this.ResizeCheckBoxM
					.setState(this.workflowPanel.getScaleOnResize());
			this.ResizeCheckBoxM.setText("Scale on Resize");
			this.ResizeCheckBoxM
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setScaleOnResize(MainWindowFrame.this.ResizeCheckBoxM
											.getState());
						}
					});
		}
		return this.ResizeCheckBoxM;
	}

	/**
	 * This method creates the displayEdgeInformationCheckBox (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getDisplayEdgeInformationCheckBox() {
		if (this.displayEdgeInformationCheckBox == null) {
			this.displayEdgeInformationCheckBox = new JCheckBoxMenuItem();
			this.displayEdgeInformationCheckBox
					.setText("Display Edge Information");
			this.displayEdgeInformationCheckBox.setState(true);
			this.displayEdgeInformationCheckBox
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setPrintInformation(
											MainWindowFrame.this.displayEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayNodeInformationCheckBox
													.getState(),
											MainWindowFrame.this.alwaysDisplayOrderKeyCheckBox
													.getState(),
											MainWindowFrame.this.displayAllEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayAllNodeInformationCheckBox
													.getState());
						}
					});
		}
		return this.displayEdgeInformationCheckBox;
	}

	/**
	 * This method creates the displayNodeInformationCheckBox (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getDisplayNodeInformationCheckBox() {
		if (this.displayNodeInformationCheckBox == null) {
			this.displayNodeInformationCheckBox = new JCheckBoxMenuItem();
			this.displayNodeInformationCheckBox
					.setText("Display Node Information");
			this.displayNodeInformationCheckBox.setState(true);
			this.displayNodeInformationCheckBox
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setPrintInformation(
											MainWindowFrame.this.displayEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayNodeInformationCheckBox
													.getState(),
											MainWindowFrame.this.alwaysDisplayOrderKeyCheckBox
													.getState(),
											MainWindowFrame.this.displayAllEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayAllNodeInformationCheckBox
													.getState());

						}
					});
		}
		return this.displayNodeInformationCheckBox;
	}

	/**
	 * This method creates the alwaysDisplayOrderKeyCheckBox (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getAlwaysDisplayOrderKeyCheckBox() {
		if (this.alwaysDisplayOrderKeyCheckBox == null) {
			this.alwaysDisplayOrderKeyCheckBox = new JCheckBoxMenuItem();
			this.alwaysDisplayOrderKeyCheckBox
					.setText("Always Display Order Keys");
			this.alwaysDisplayOrderKeyCheckBox.setState(false);
			this.alwaysDisplayOrderKeyCheckBox
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setPrintInformation(
											MainWindowFrame.this.displayEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayNodeInformationCheckBox
													.getState(),
											MainWindowFrame.this.alwaysDisplayOrderKeyCheckBox
													.getState(),
											MainWindowFrame.this.displayAllEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayAllNodeInformationCheckBox
													.getState());
						}
					});
		}
		return this.alwaysDisplayOrderKeyCheckBox;
	}

	/**
	 * This method creates the displayAllEdgeInformationCheckbox (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getDisplayAllEdgeInformationCheckBox() {
		if (this.displayAllEdgeInformationCheckBox == null) {
			this.displayAllEdgeInformationCheckBox = new JCheckBoxMenuItem();
			this.displayAllEdgeInformationCheckBox
					.setText("Display All Edge Information");
			this.displayAllEdgeInformationCheckBox.setState(false);
			this.displayAllEdgeInformationCheckBox
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setPrintInformation(
											MainWindowFrame.this.displayEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayNodeInformationCheckBox
													.getState(),
											MainWindowFrame.this.alwaysDisplayOrderKeyCheckBox
													.getState(),
											MainWindowFrame.this.displayAllEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayAllNodeInformationCheckBox
													.getState());

						}
					});
		}
		return this.displayAllEdgeInformationCheckBox;
	}

	/**
	 * This method creates the displayAllNodeInformationCheckBox (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getDisplayAllNodeInformationCheckBox() {
		if (this.displayAllNodeInformationCheckBox == null) {
			this.displayAllNodeInformationCheckBox = new JCheckBoxMenuItem();
			this.displayAllNodeInformationCheckBox
					.setText("Display All Node Information");
			this.displayAllNodeInformationCheckBox.setState(false);
			this.displayAllNodeInformationCheckBox
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setPrintInformation(
											MainWindowFrame.this.displayEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayNodeInformationCheckBox
													.getState(),
											MainWindowFrame.this.alwaysDisplayOrderKeyCheckBox
													.getState(),
											MainWindowFrame.this.displayAllEdgeInformationCheckBox
													.getState(),
											MainWindowFrame.this.displayAllNodeInformationCheckBox
													.getState());

						}
					});
		}
		return this.displayAllNodeInformationCheckBox;
	}

	/**
	 * This method creates the ShowGridCheckBoxM (if necessary) and returns it.
	 */
	private JCheckBoxMenuItem getShowGridCheckBoxM() {
		if (this.ShowGridCheckBoxM == null) {
			this.ShowGridCheckBoxM = new JCheckBoxMenuItem();
			this.ShowGridCheckBoxM.setText("Display Grid");
			this.ShowGridCheckBoxM.setState(true);
			this.ShowGridCheckBoxM
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							MainWindowFrame.this.workflowPanel
									.setShowGrid(MainWindowFrame.this.ShowGridCheckBoxM
											.getState());
						}
					});
			this.workflowPanel.setShowGrid(this.ShowGridCheckBoxM.getState());
		}
		return this.ShowGridCheckBoxM;
	}

	/**
	 * This method returns if the checkbox "Show Grid" is marked or not.
	 */
	public boolean getShowGridStatus() {
		return this.ShowGridCheckBoxM.getState();
	}

	/**
	 * This method opens the workflow specified by the passed filename.
	 * 
	 * @param checkFilesOnLoad
	 * @param checkSchema
	 */
	public void openGraph(String pFileName, boolean checkFilesOnLoad) {
		this.workflowPanel.loadGraph(pFileName, checkFilesOnLoad,
				this.checkSchema);
	}

	/**
	 * This method creates the helpContentItem (if necessary) and returns it.
	 */
	private JMenuItem getHelpContentItem() {
		if (this.helpContentItem == null) {
			this.helpContentItem = new JMenuItem("Help");
			this.helpContentItem.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					HelpFrame helpFrame = new HelpFrame(450, 300);
					helpFrame.setLocationRelativeTo(me);
					Point r = helpFrame.getLocation();
					r.move(r.x + 240, r.y + 180);
					helpFrame.setLocation(r);
					helpFrame.setVisible(true);
				}
			});
		}
		return this.helpContentItem;
	}

	/**
	 * This method creates the aboutItem (if necessary) and returns it.
	 */
	private JMenuItem getAboutItem() {
		if (this.aboutItem == null) {
			this.aboutItem = new JMenuItem("About ProFiDo");
			this.aboutItem.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					AboutFrame aboutFrame = new AboutFrame(700, 710);
					aboutFrame.setLocationRelativeTo(me);
					Point r = aboutFrame.getLocation();
					r.move(r.x + 240, r.y + 180);
					aboutFrame.setLocation(r);
					aboutFrame.setVisible(true);

				}
			});
		}
		return this.aboutItem;
	}

	/**
	 * This method is used to export the current Workflow as a script.
	 */
	public void exportWorkflowScript(String exportFilenName) {
		this.workflowPanel.exportWorkflowScript(exportFilenName, false);
	}

	public WorkflowPanel getWorkflowPanel() {
		return this.workflowPanel;
	}

	public void setCheckSchema(boolean checkSchema) {
		this.checkSchema = checkSchema;
	}

}
