/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import helper.SmallHelpers;

import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JLabel;
import javax.swing.JPanel;

import core.storage.ConfigurationXMLReader;

/**
 * This class defines the properties of main panel of the gui.
 * 
 * The MainWindowPanel consists of the WorkflowPanel and two Jlabels which are
 * used by the WorkflowPanel to display further information.
 */

public class MainWindowPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Label for displaying status information about the workflow.
	 */
	private JLabel statusLabel = null;

	/**
	 * Label for displaying additional information about the workflow.
	 */
	private JLabel infoLabel = null;

	/**
	 * Reference to the WorkflowPanel
	 */
	private WorkflowPanel workflowPanel = null;

	/**
	 * This is the default constructor
	 */
	public MainWindowPanel(ConfigurationXMLReader xmlReader,
			MainWindowFrame mainFrame) {
		super();
		this.initialize(xmlReader, mainFrame);
	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		int labelHeight = 16;
		int labelTopSpace = 4;
		if (this.workflowPanel != null) {
			this.workflowPanel.setBounds(new Rectangle(0, 0, this.getWidth(),
					this.getHeight() - (labelHeight + labelTopSpace)));
		}
		if (this.statusLabel != null) {
			this.statusLabel.setBounds(new Rectangle(1, this.getHeight()
					- labelHeight, 400, labelHeight));
		}
		if (this.infoLabel != null) {
			this.infoLabel.setBounds(new Rectangle(
					this.statusLabel.getWidth() + 5, this.getHeight()
							- labelHeight, 200, labelHeight));
		}
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize(ConfigurationXMLReader xmlReader,
			MainWindowFrame mainFrame) {
		this.infoLabel = new JLabel();
		this.infoLabel.setText("");
		this.statusLabel = new JLabel();
		this.statusLabel.setText("");
		this.setSize(300, 300); // ?
		this.setLayout(null);
		this.add(this.statusLabel, null);
		this.add(this.infoLabel, null);
		this.add(this.getWorkflowPanel(xmlReader, mainFrame), null);
		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				MainWindowPanel.this.updatePositions();
			}
		});
	}

	/**
	 * This method creates the WorkflowPanel for a given ConfigurationXMLReader
	 * and a MainWindowsFrame.
	 */
	public WorkflowPanel getWorkflowPanel(ConfigurationXMLReader xmlReader,
			MainWindowFrame mainFrame) {
		if (this.workflowPanel == null) {
			this.workflowPanel = new WorkflowPanel(this.statusLabel,
					this.infoLabel, xmlReader, mainFrame);
			SmallHelpers.workflowPanel = this.workflowPanel;
		}
		return this.workflowPanel;
	}

	/**
	 * Returns the name of the workflow displayed by this panels WorkflowPanel.
	 */
	public String getWorkflowName() {
		return this.workflowPanel.getBatchName();
	}

}
