/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package helper;

import java.awt.Component;
import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import core.ParameterGroup;

/**
 * This class defines several method used by other classes.
 */
public class SmallHelpers {
	
	/**
	 * Reference to the current WorkFlowPanel.
	 */
	public static Component workflowPanel;
	
	/**
	 * This String represents all valid characters.
	 */
	public static String characterRegExp = "([\\w\\s]|[\\}\\{]|[\\\"\\$\\.\\-\\+\\(\\)\\_\\/\\\\])";


	/**
	 * This method escapes your a given String into a Latex conform String.
	 */
	public static String escapeLatex(String input) {
		return input.replace("_", "\\_").replace("$", "\\$").replace("#", "\\#");
	}
	
	/**
	 * This method is needed to create Latex conform Id's.
	 */
	public static String removeForLatexId(String input){
		
		return input.replace("_", "").replace("$", "").replace("#", "");
	}

	/**
	 * This method checks if a Char is valid.
	 */
	public static boolean isCharValid(char charToCheck, String additionalPattern) {
		String toCheck = String.valueOf(charToCheck); // Convert to string
		if (!additionalPattern.equals(""))
			return toCheck.matches(SmallHelpers.characterRegExp)
					|| toCheck.matches(additionalPattern);
		else
			return toCheck.matches(SmallHelpers.characterRegExp);
	}

	/**
	 * This method checks if a String is valid.
	 */
	public static boolean isStringValid(String stringToCheck,
			String additionalPattern) {
		for (int curPos = 0; curPos < stringToCheck.length(); curPos++) {
			if (!SmallHelpers.isCharValid(stringToCheck.charAt(curPos),
					additionalPattern))
				return false;
		}

		return true;
	}

	/**
	 * This method is used to remove all invalid chars of a String.
	 */
	public static String removeAllInvalidChars(String input,
			String additionalPattern) {
		String result = "";
		for (int curPos = 0; curPos < input.length(); curPos++) {
			if (SmallHelpers.isCharValid(input.charAt(curPos),
					additionalPattern)) {
				result = result + input.charAt(curPos);
			} else {
			}
		}

		return result;
	}

	/**
	 * This method is used to remove all invalid chars of a String and
	 * creates a warning message if its needed.
	 */
	public static String removeAllInvalidChars(String input,
			String additionalPattern, String warningInformation) {
		String newString = SmallHelpers.removeAllInvalidChars(input,
				additionalPattern);
		if (!newString.equals(input)) {
			JOptionPane
					.showMessageDialog(
							SmallHelpers.getWorkflowPanel(),
							"The "
									+ warningInformation
									+ " "
									+ input
									+ " contains invalid characters.\nAll invalid characters have been removed.",
							"Warning", JOptionPane.WARNING_MESSAGE);
		}

		return newString;
	}

	/**
	 * Converts an array of strings into a vector and returns it.
	 */
	public static Vector<String> arrayToVector(String[] args) {
		Vector<String> result = new Vector<String>();
		for (String arg : args) {
			result.add(arg);
		}
		return result;
	}

	/**
	 * This method returns the parameter group from the passed vector which
	 * matches the passed name.
	 */
	public static ParameterGroup findParameterGroupByName(String targetName,
			Vector<ParameterGroup> groups) {
		for (ParameterGroup curParameterGroup : groups) {
			if (curParameterGroup.getName().equals(targetName))
				return curParameterGroup;
		}

		return null;
	}

	/**
	 * Checks if a given file exists and can be read by the user. If a $HOME
	 * variable is found, it is replaced before performing the above mentioned
	 * checks.
	 */
	public static boolean fileReadable(File orgFile) {
		File file = new File(SmallHelpers.replaceVariables(orgFile.getPath()));
		return file.canRead() && file.exists();
	}

	/**
	 * Checks if a given file can be written by the user. If a $HOME variable is
	 * found, it is replaced before performing the above mentioned checks.
	 */
	public static boolean fileWriteable(File orgFile) {
		File file = new File(SmallHelpers.replaceVariables(orgFile.getPath()));
		return file.getParentFile().canWrite();
	}

	/**
	 * Replaces any bash-style variables found in the passed string with the
	 * corresponding values.
	 */
	public static String replaceVariables(String fileName) {
		String result = fileName.replace("${HOME}",
				System.getProperty("user.home"));
		return result;
	}

	/**
	 * This method is used to merge two Vectors.
	 */
	public static Vector<String> intersect(Vector<String> a, Vector<String> b) {
		if (a == null)
			return b;
		else if (b == null)
			return a;
		else {
			Vector<String> result = new Vector<String>();
			for (String curA : a) {
				boolean found = false;
				for (String curB : b) {
					if (curA.equals(curB)) {
						found = true;
						break;
					}
				}
				if (found) {
					result.add(curA);
				}
			}

			return result;
		}
	}

	public static String ensureValidPath(String inputPath) {
		inputPath = inputPath.replace('\\', '/');
		while (inputPath.indexOf("//") != -1) {
			inputPath = inputPath.replace("//", "/");
		}
		return inputPath;
	}

	/**
	 * This method returns the current WorkflowPanel.
	 */
	public static Component getWorkflowPanel() {
		return SmallHelpers.workflowPanel;
	}

}
