/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package helper.testmanager;

import gui.MainWindowFrame;

import java.awt.AWTException;
import java.awt.Point;
import java.awt.Robot;
import java.util.Random;

import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * The TestManager superclass provides a standardised architecture which allows
 * the simulation of any desired user interaction. In combination with ProFiDo's
 * ability to load a workflow from the location specified by its first command
 * line parameter and export it to the location specified by its second, a
 * series of tests can be automated by a bash script which is described in the
 * ProFiDo manual.
 */
public abstract class TestManager {
	
	/**
	 * Robot that can be used to perform clicks on the gui.
	 */
	protected Robot testRobot;

	/**
	 * Upper left point of this TestManagers target panel in screen coordinates.
	 */
	protected Point targetPanelUpperLeftInScreenCoordinates;

	/**
	 * Reference to the panel that will be tested by this TestManager.
	 */
	protected JPanel targetPanel;

	/**
	 * ProFiDos MainWindow.
	 */
	protected MainWindowFrame rootWindow;

	/**
	 * Random number generator that can be used when needed.
	 */
	protected Random random;

	/**
	 * Returns a random number between 0 and n - 1.
	 */
	public int getRandom(int n) {
		return Math.abs(this.random.nextInt() % n);
	}

	/**
	 * Constructor to create a TestManager.
	 */
	public TestManager(MainWindowFrame window, JPanel targetPanel, long seed)
			throws AWTException {
		this.random = new Random(seed);

		this.testRobot = new Robot();
		this.targetPanel = targetPanel;
		this.rootWindow = window;

		Point testedPanelUpperLeftInRootCoordiantes = SwingUtilities
				.convertPoint(targetPanel, new Point(0, 0), window);
		int testAreaUpperLeftScreenX = (int) Math
				.round(testedPanelUpperLeftInRootCoordiantes.getX()
						+ window.getX());
		int testAreaUpperLeftScreenY = (int) Math
				.round(testedPanelUpperLeftInRootCoordiantes.getY()
						+ window.getY());
		this.targetPanelUpperLeftInScreenCoordinates = new Point(
				testAreaUpperLeftScreenX, testAreaUpperLeftScreenY);
	}

	/**
	 * Performs the specified amount of tests on this TestManagers assigned part
	 * of the GUI.
	 */
	public abstract void performTests(int testInteractionCount, int testDelay);
}
