/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package helper.testmanager;

import gui.MainWindowFrame;
import gui.WorkflowPanel;

import java.awt.AWTException;
import java.awt.Point;
import java.awt.event.InputEvent;
import java.util.Vector;

import javax.swing.JPanel;

/**
 * This subclass tests the main GUI by performing a series of random
 * modifications (deleting nodes and edges) to the workflow and undoing these
 * changes afterwards. The nodes and edges that will be deleted are randomly
 * selected, using the time since midnight, January 1, 1970 UTC as seed for the
 * random number generator.
 */
public class WorkflowPanelTestManager extends TestManager {
	
	/**
	 * Constructor for a WorkflowPanelTextManager object.
	 */
	public WorkflowPanelTestManager(MainWindowFrame window, JPanel targetPanel,
			long seed) throws AWTException {
		super(window, targetPanel, seed);
	}

	@Override
	public void performTests(int testInteractionCount, int testDelay) {
		
		/**
		 * How many Jobs should be deleted.
		 */
		int jobsToDelete = testInteractionCount / 2;
		
		/**
		 * How many Edges should be deleted.
		 */
		int edgesToDelete = testInteractionCount - jobsToDelete;

		// Ensure that the amount of deleted jobs and endges is limited by the
		// acutal amount of nodes and edges
		int jobCount = ((WorkflowPanel) this.targetPanel).getJobNodePositions()
				.size();
		int edgeCount = ((WorkflowPanel) this.targetPanel).getEdgePositions()
				.size();
		if (jobsToDelete > jobCount) {
			jobsToDelete = jobCount;
		}
		if (edgesToDelete > edgeCount) {
			edgesToDelete = edgeCount;
		}

		// Delete jobs
		for (int i = 0; i < jobsToDelete; i++) {
			this.deleteRandomJob(testDelay);
			this.delay(testDelay);
		}

		// Restore jobs
		for (int i = 0; i < jobsToDelete; i++) {
			this.rootWindow.getUndoMenuItem().doClick();
			// delay(testDelay);
		}

		// Delete edges
		for (int i = 0; i < edgesToDelete; i++) {
			this.deleteRandomEdge(testDelay);
			this.delay(testDelay);
		}

		// Restore edges
		for (int i = 0; i < edgesToDelete; i++) {
			this.rootWindow.getUndoMenuItem().doClick();
			// delay(testDelay);
		}
	}

	/**
	 * This method is called to delay the Thread between a robot action. 
	 */
	private void delay(int testDelay) {
		try {
			Thread.sleep(testDelay);
		} catch (InterruptedException e) {
		}

	}

	/**
	 * This method is called to delete random jobs.
	 */
	public void deleteRandomJob(int testDelay) {
		Vector<Point> jobPositions = ((WorkflowPanel) this.targetPanel)
				.getJobNodePositions();
		if (jobPositions.size() > 0) {
			Point targetJobLocation = jobPositions.get(this
					.getRandom(jobPositions.size()));
			this.deleteNodeAtPosition(targetJobLocation.x, targetJobLocation.y,
					"job", testDelay);
		}
	}

	/**
	 * This method deletes random edges.
	 */
	public void deleteRandomEdge(int testDelay) {
		Vector<Point> edgePositions = ((WorkflowPanel) this.targetPanel)
				.getEdgePositions();
		if (edgePositions.size() > 0) {
			Point targetEdgePosition = edgePositions.get(this
					.getRandom(edgePositions.size()));
			this.deleteNodeAtPosition(targetEdgePosition.x,
					targetEdgePosition.y, "edge", testDelay);
		}
	}

	/**
	 * This method deletes a Node at the position (curX,curY).
	 */
	private void deleteNodeAtPosition(int curX, int curY, String type,
			int testDelay) {
		try {
			this.testRobot.mouseMove(curX
					+ this.targetPanelUpperLeftInScreenCoordinates.x, curY
					+ this.targetPanelUpperLeftInScreenCoordinates.y);
			this.testRobot.mousePress(InputEvent.BUTTON3_MASK);
			this.testRobot.mouseRelease(InputEvent.BUTTON3_MASK);
			this.delay(testDelay);
			if (type.equals("job")) {
				((WorkflowPanel) this.targetPanel).getDeleteJobMenuItem()
						.doClick();
			} else if (type.equals("edge")) {
				((WorkflowPanel) this.targetPanel).getDeleteEdgeMenuItem()
						.doClick();
			}
			this.testRobot.mouseMove(curX
					+ this.targetPanelUpperLeftInScreenCoordinates.x - 10, curY
					+ this.targetPanelUpperLeftInScreenCoordinates.y - 10);
			this.testRobot.mousePress(InputEvent.BUTTON1_MASK);
			this.testRobot.mouseRelease(InputEvent.BUTTON1_MASK);
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
